﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buddy.Common.Plugins;
using Buddy.Common;
using Buddy.Swtor;
using Buddy.Swtor.Objects;
using System.Diagnostics;
using System.IO;
using System.Windows;

namespace ProfileBuilder
{
    class ProfileBuilder : IPlugin
    {
        public string Author { get { return "Ama"; } }
        public Version Version { get { return new Version(1, 0); } }
        public string Name { get { return "ProfileBuilder"; } }
        public string Description { get { return "Please Work."; } }
        public Window DisplayWindow { get { return null; } }

        public static bool recording = false;
        public static string ProfileName = "myProfile";
        public static int interval = 3;
        public static int pointSeparation = 12;
        public static int startLevel = 0;
        public static int endLevel = 50;
        public static Stopwatch sw = new Stopwatch();
        public static List<GrindPoint> Medics = new List<GrindPoint>();
        public static List<GrindPoint> HotSpots = new List<GrindPoint>();
        protected static Buddy.Swtor.Objects.TorPlayer Me { get { return BuddyTor.Me; } }
        public PBForm frm;
        public static string outputPath = Path.Combine(Directory.GetCurrentDirectory(), string.Format(@"Profiles\" + ProfileName + ".xml"));

        public bool Equals(IPlugin other)
        {
            return other.Name == Name;
        }

        public void OnInitialize()
        {
            if(!sw.IsRunning)
                sw.Start();
        }

        public void OnPulse()
        {
            if (recording && sw.IsRunning && sw.Elapsed.TotalSeconds >= interval)
            {
                sw.Stop();
                sw.Reset();
                sw.Start();
                addHotSpots();
            }
        }

        public void OnEnabled()
        {
            frm = new PBForm();
            frm.Show();
        }

        public void OnDisabled()
        {
            frm.Dispose();
            frm = null;
        }

        public void OnShutdown()
        {
            if (frm != null)
            {
                frm.Close();
                frm = null;
            }
        }

        public void addHotSpots()
        {
            GrindPoint gp;
            foreach(TorCharacter c in ObjectManager.GetObjects<TorCharacter>()){
                gp = new GrindPoint(c);
                if (c.Name.Contains("Medical Droid") && !c.IsHostile && (GoodPoint(gp, Medics)))
                {
                    Medics.Add(gp);
                    Logging.Write("ProfileBuilder Added: " + gp.MobName);
                }

                if (c.IsHostile && c.IsTargetable && !c.IsNeutral && (int)c.Level >= startLevel && (int)c.Level <= endLevel && (GoodPoint(gp, HotSpots)))
                { 
                    HotSpots.Add(gp);
                    Logging.Write("ProfileBuilder Added: " + gp.MobName);
                }
            }
        }

        public bool GoodPoint(GrindPoint g, List<GrindPoint> list)
        {
            foreach (GrindPoint hotspot in list)
            {
                if (Distance(hotspot, g) < pointSeparation)
                    return false;
            }
            return true;
        }

        public int Distance(GrindPoint p1, GrindPoint p2)
        {
            double XDif = Math.Pow((p2.X - p1.X), 2);
            double YDif = Math.Pow((p2.Y - p1.Y), 2);
            int distance = (int)Math.Sqrt(XDif + YDif);

            return distance;
        }

        public static List<string> mobNames()
        {
            List<string> list = new List<string>();
            foreach (GrindPoint g in HotSpots)
            {
                if (!list.Contains(g.MobName))
                    list.Add(g.MobName);
            }
            return list;
        }

        public static void OutputProfile()
        {
            try
            {
                DirectoryInfo dir = new DirectoryInfo(Path.Combine(Directory.GetCurrentDirectory(), string.Format(@"Profiles\")));
                if (!dir.Exists)
                    dir.Create();

                outputPath = Path.Combine(Directory.GetCurrentDirectory(), string.Format(@"Profiles\" + ProfileName + ".xml"));

                StreamWriter o = new StreamWriter(outputPath);
                Logging.Write(outputPath);

                o.WriteLine("<Profile xmlns=\"http://tempuri.org/ProfileSchema.xsd\">");
                o.WriteLine("\t<Name>" + ProfileName + "</Name>");
                o.WriteLine("");
                //Medics
                o.WriteLine("\t\t<Vendors>");
                foreach (GrindPoint g in Medics)
                {
                    o.WriteLine("\t\t\t" + g.BuildString());
                }
                o.WriteLine("\t\t</Vendors>");
                o.WriteLine("");

                //GrindAreas
                o.WriteLine("\t\t<GrindAreas>");
                o.WriteLine("\t\t\t<GrindArea Name=\"" + HotSpots.ElementAt(0).MobName + "\">\"");
                o.WriteLine("\t\t\t\t<MinLevel>" + startLevel + "</MinLevel>");
                o.WriteLine("\t\t\t\t<MaxLevel>" + endLevel + "</MaxLevel>");
                
                //Mob Names
                o.WriteLine("\t\t\t\t<Mobs>");
                foreach (String s in mobNames())
                {
                    o.WriteLine("\t\t\t\t\t<Mob>" + s + "</Mob>");
                }
                o.WriteLine("\t\t\t\t</Mobs>");

                //HotSpots
                o.WriteLine("\t\t\t\t<Hotspots>");
                foreach (GrindPoint g in HotSpots)
                {
                    o.WriteLine("\t\t\t\t\t" + g.BuildString());
                }
                o.WriteLine("\t\t\t\t</Hotspots>");
                o.WriteLine("\t\t\t</GrindArea>");
                o.WriteLine("\t\t</GrindAreas>");
                o.WriteLine("\t\t\t\t<Grind GrindRef=\"" + HotSpots.ElementAt(0).MobName + "\" While=\"Me.Level &lt; " + endLevel + "\" />");
                o.WriteLine("</Profile>");
                o.Close();
            }
            catch
            {
                Logging.Write(outputPath);
            }
        }
    }
}
